@@ -1,6 +1,7 @@
 /* Dirty system-dependent hacks.
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+   Inc.
 
 This file is part of GNU Wget.
 
@@ -70,20 +71,6 @@ as that of the covered work.  */
 /* Request the "Unix 98 compilation environment". */
 #define _XOPEN_SOURCE 500
 
-/* For Solaris: request everything else that is available and doesn't
-   conflict with the above.  */
-/* #define __EXTENSIONS__ */ /* XXX clashes with config.h */
-
-/* For Linux: request features of 4.3BSD and SVID (System V Interface
-   Definition). */
-#define _SVID_SOURCE
-#define _BSD_SOURCE
-
-/* Under glibc-based systems we want all GNU extensions as well.  This
-   declares some unnecessary cruft, but also useful functions such as
-   timegm, FNM_CASEFOLD extension to fnmatch, memrchr, etc.  */
-/* #define _GNU_SOURCE */ /* XXX clashes with config.h */
-
 #endif /* NAMESPACE_TWEAKS */
 
 
@@ -91,35 +78,15 @@ as that of the covered work.  */
    These have to be after the above namespace tweaks, but before any
    non-preprocessor code.  */
 
-#if HAVE_ALLOCA_H
-# include <alloca.h>
-#elif defined WINDOWS
-# include <malloc.h>
-# ifndef alloca
-#  define alloca _alloca
-# endif
-#elif defined __GNUC__
-# define alloca __builtin_alloca
-#elif defined _AIX
-# define alloca __alloca
-#else
-# include <stddef.h>
-# ifdef  __cplusplus
-extern "C"
-# endif
-void *alloca (size_t);
-#endif
+#include <alloca.h>
 
 /* Must include these, so we can test for the missing stat macros and
    define them as necessary.  */
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#ifdef HAVE_INTTYPES_H
-  /* Compaq C V6.5-303 (dtk) on HP Tru64 UNIX V5.1B (Rev. 2650) needs: */
-# include <stdint.h>
-# include <inttypes.h>
-#endif
+#include <stdint.h>
+#include <inttypes.h>
 
 #ifdef WINDOWS
 /* Windows doesn't have some functions normally found on Unix-like
@@ -131,52 +98,9 @@ void *alloca (size_t);
 # include "mswindows.h"
 #endif
 
-/* Provide support for C99-type boolean type "bool".  This blurb comes
-   straight from the Autoconf 2.59 manual. */
-#if HAVE_STDBOOL_H
+/* Provided by gnulib on systems that don't have it: */
 # include <stdbool.h>
-#else
-# if ! HAVE__BOOL
-#  ifdef __cplusplus
-typedef bool _Bool;
-#  else
-typedef unsigned char _Bool;
-#  endif
-# endif
-# define bool _Bool
-# define false 0
-# define true 1
-# define __bool_true_false_are_defined 1
-#endif
 
-/* Needed for compilation under OS/2 and MSDOS */
-#if defined(__EMX__) || defined(MSDOS)
-# ifndef S_ISLNK
-#  define S_ISLNK(m) 0
-# endif
-# ifndef lstat
-#  define lstat stat
-# endif
-#endif /* __EMX__ || MSDOS */
-
-/* Reportedly, stat() macros are broken on some old systems.  Those
-   systems will have to fend for themselves, as I will not introduce
-   new code to handle it.
-
-   However, I will add code for *missing* macros, and the following
-   are reportedly missing from many systems.  */
-#ifndef S_ISLNK
-# define S_ISLNK(m) (((m) & S_IFMT) == S_IFLNK)
-#endif
-#ifndef S_ISDIR
-# define S_ISDIR(m) (((m) & (_S_IFMT)) == (_S_IFDIR))
-#endif
-#ifndef S_ISREG
-# define S_ISREG(m) (((m) & _S_IFMT) == _S_IFREG)
-#endif
-
-/* These are needed so we can #define struct_stat to struct _stati64
-   under Windows. */
 #ifndef struct_stat
 # define struct_stat struct stat
 #endif
@@ -184,6 +108,8 @@ typedef unsigned char _Bool;
 # define struct_fstat struct stat
 #endif
 
+#include <intprops.h>
+
 /* For CHAR_BIT, LONG_MAX, etc. */
 #include <limits.h>
 
@@ -191,9 +117,6 @@ typedef unsigned char _Bool;
 # define CHAR_BIT 8
 #endif
 
-/* From gnulib, simplified to assume a signed type. */
-#define TYPE_MAXIMUM(t) ((t) (~ (~ (t) 0 << (sizeof (t) * CHAR_BIT - 1))))
-
 /* These are defined in cmpt.c if missing, so we must declare
    them.  */
 #ifndef HAVE_STRCASECMP
@@ -216,10 +139,11 @@ void *memrchr (const void *, int, size_t);
 /* These are defined in snprintf.c.  It would be nice to have an
    snprintf.h, though.  */
 #ifndef HAVE_SNPRINTF
-int snprintf ();
+int snprintf (char *str, size_t count, const char *fmt, ...);
 #endif
 #ifndef HAVE_VSNPRINTF
-int vsnprintf ();
+#include <stdarg.h>
+int vsnprintf (char *str, size_t count, const char *fmt, va_list arg);
 #endif
 
 /* Some systems (Linux libc5, "NCR MP-RAS 3.0", and others) don't
@@ -239,23 +163,7 @@ int vsnprintf ();
 # define SYSTEM_FNMATCH
 #endif
 
-#ifdef SYSTEM_FNMATCH
-# include <fnmatch.h>
-#else  /* not SYSTEM_FNMATCH */
-/* Define fnmatch flags.  Undef them first to avoid warnings in case
-   an evil library include chose to include system fnmatch.h.  */
-# undef FNM_PATHNAME
-# undef FNM_NOESCAPE
-# undef FNM_PERIOD
-# undef FNM_NOMATCH
-
-# define FNM_PATHNAME	(1 << 0) /* No wildcard can ever match `/'.  */
-# define FNM_NOESCAPE	(1 << 1) /* Backslashes don't quote special chars.  */
-# define FNM_PERIOD	(1 << 2) /* Leading `.' is matched only explicitly.  */
-# define FNM_NOMATCH	1
-
-int fnmatch (const char *, const char *, int);
-#endif
+#include <fnmatch.h>
 
 /* Provide sig_atomic_t if the system doesn't.  */
 #ifndef HAVE_SIG_ATOMIC_T
